@@ -3,20 +3,20 @@ Introduction
 ============
 
 grib2io is a Python package that provides an interface to the [NCEP GRIB2 C (g2c)](https://github.com/NOAA-EMC/NCEPLIBS-g2c) 
-library for the purpose of reading and writing GRIB2 Messages.  WMO GRIdded Binary, Edition 2 (GRIB2) files store 2-D meteorological
-data. A physical file can contain one or more GRIB2 messages.  File IO is handled in Python returning
-a binary string of the GRIB2 message which is then passed to the g2c library for decoding of GRIB2 metadata
-and unpacking of data values.
+library for the purpose of reading and writing WMO GRIdded Binary, Edition 2 (GRIB2) messages. A physical file can contain one 
+or more GRIB2 messages.
+
+GRIB2 file IO is performed directly in Python.  The unpacking/packing of GRIB2 integer, coded metadata and data sections is performed 
+by the g2c library functions via the g2clib Cython wrapper module.  The decoding/encoding of GRIB2 metadata is translated into more 
+descriptive, plain language metadata by looking up the integer code values against the appropriate GRIB2 code tables.  These code tables 
+are a part of the grib2io module.
 """
-__version__ = '0.9.1'
 
-import g2clib
 import builtins
 import collections
 import copy
 import datetime
 import os
-#import pdb
 import re
 import struct
 import math
@@ -26,11 +26,13 @@ from numpy import ma
 import numpy as np
 import pyproj
 
+from . import g2clib
 from . import tables
 from . import utils
 
-
+DEFAULT_DRT_LEN = 20
 DEFAULT_FILL_VALUE = 9.9692099683868690e+36
+DEFAULT_NUMPY_INT = np.int64
 GRIB2_EDITION_NUMBER = 2
 ONE_MB = 1048576 # 1 MB in units of bytes
 
@@ -59,11 +61,16 @@ class open():
 
     **`closed`** `True` is file handle is close; `False` otherwise.
 
+    **`decode `**: If `True` [DEFAULT] automatically decode metadata from unpacked 
+    section data for Grib2Messages.
+
     **`variables`**: Tuple containing a unique list of variable short names (i.e. GRIB2 abbreviation names).
+
+    **`levels`**: Tuple containing a unique list of wgrib2-formatted level/layer strings.
     """
-    def __init__(self, filename, mode='r'):
+    def __init__(self, filename, mode='r', decode=True):
         """
-        Class Constructor
+        `open` Constructor
 
         Parameters
         ----------
@@ -72,6 +79,9 @@ class open():
 
         **`mode `**: File access mode where `r` opens the files for reading only;
         `w` opens the file for writing.
+
+        **`decode `**: If `True` [DEFAULT] automatically decode metadata from 
+        unpacked section data for Grib2Messages.
         """
         if mode in ['a','r','w']:
             mode = mode+'b'
@@ -84,12 +94,13 @@ class open():
         self.current_message = 0
         self.size = os.path.getsize(self.name)
         self.closed = self._filehandle.closed
-        self.decode = True
+        self.decode = decode
         if 'r' in self.mode: self._build_index()
         # FIX: Cannot perform reads on mode='a'
         #if 'a' in self.mode and self.size > 0: self._build_index()
         if self._hasindex:
             self.variables = tuple(sorted(set(filter(None,self._index['shortName']))))
+            self.levels = tuple(sorted(set(filter(None,self._index['levelString']))))
 
 
     def __delete__(self, instance):
@@ -183,6 +194,8 @@ class open():
         self._index['duration'] = [None]
         self._index['shortName'] = [None]
         self._index['bitMap'] = [None]
+        self._index['levelString'] = [None]
+        self._index['probString'] = [None]
 
         # Iterate
         while True:
@@ -275,6 +288,11 @@ class open():
                             self._index['duration'].append(utils.getduration(_pdtnum,_pdt))
                             self._index['shortName'].append(_varinfo[2])
                             self._index['bitMap'].append(_bmap)
+                            self._index['levelString'].append(tables.get_wgrib2_level_string(*_pdt[9:15]))
+                            if _pdtnum in [5,9]:
+                                self._index['probString'].append(utils.get_wgrib2_prob_string(*_pdt[17:22]))
+                            else:
+                                self._index['probString'].append('')
                             if _issubmessage:
                                 self._index['submessageOffset'].append(_submsgoffset)
                                 self._index['submessageBeginSection'].append(_submsgbegin)
@@ -299,12 +317,15 @@ class open():
                             self._index['duration'].append(utils.getduration(_pdtnum,_pdt))
                             self._index['shortName'].append(_varinfo[2])
                             self._index['bitMap'].append(_bmap)
+                            self._index['levelString'].append(tables.get_wgrib2_level_string(*_pdt[9:15]))
+                            if _pdtnum in [5,9]:
+                                self._index['probString'].append(utils.get_wgrib2_prob_string(*_pdt[17:22]))
+                            else:
+                                self._index['probString'].append('')
                             self._index['submessageOffset'].append(_submsgoffset)
                             self._index['submessageBeginSection'].append(_submsgbegin)
                             continue
 
-                #print(len(self._index['offset']),self.messages)
-
             except(struct.error):
                 self._filehandle.seek(0)
                 break
@@ -312,50 +333,6 @@ class open():
         self._hasindex = True
 
 
-    def _find_level(self,level):
-        """
-        """
-        # Determine level or layer....TBD
-        #
-        # IMPORTANT: Set idx_count to the number of searches.
-        if any(re.findall(r'ground|surface|sfc', level, re.IGNORECASE)):
-            # Ground/Surface - GRIB ID = 1
-            sfctypeid = 1
-            idx_type = np.where(np.asarray([i[9] if i is not None else None for i in self._index['productDefinitionTemplate']])==sfctypeid)[0]
-            idxs = idx_type
-            idx_count = 1
-        elif any(re.findall(r'mb|pa|hpa', level, re.IGNORECASE)):
-            # Isobaric Surface (i.e. pressure level) - GRIB ID = 100
-            sfctypeid = 100
-            idx_type = np.where(np.asarray([i[9] if i is not None else None for i in self._index['productDefinitionTemplate']])==sfctypeid)[0]
-            val = float(re.sub("[^\d\.]", "",level))
-            if any(re.findall(r'mb|hpa', level, re.IGNORECASE)): val *= 100
-            idx_val = np.where(np.asarray([i[11] if i is not None else None for i in self._index['productDefinitionTemplate']])==val)[0]
-            idxs = np.concatenate((idx_type,idx_val))
-            idx_count = 2
-        elif any(re.findall(r'sig|sigma', level, re.IGNORECASE)):
-            # Sigma Level - GRIB ID = 104
-            sfctypeid = 104
-            idx_type = np.where(np.asarray([i[9] if i is not None else None for i in self._index['productDefinitionTemplate']])==sfctypeid)[0]
-            val = float(re.sub("[^\d\.]", "",level))
-            idx_val = np.where(np.asarray([i[11]/(10**i[10]) if i is not None else None for i in self._index['productDefinitionTemplate']])==val)[0]
-            idxs = np.concatenate((idx_type,idx_val))
-            idx_count = 2
-        elif any(re.findall(r'm|meter', level, re.IGNORECASE)):
-            # Specified Height Level Above (GRIB ID = 103) or Below Ground (GRIB ID = 106) Level
-            sfctypeid = 103
-            if any(re.findall(r'above ground|agl', level, re.IGNORECASE)):
-                sfctypeid = 103
-            if any(re.findall(r'below ground|bgl', level, re.IGNORECASE)):
-                sfcid = 106
-            idx_type = np.where(np.asarray([i[9] if i is not None else None for i in self._index['productDefinitionTemplate']])==sfctypeid)[0]
-            val = float(re.sub("[^\d\.]", "",level))
-            idx_val = np.where(np.asarray([i[11] if i is not None else None for i in self._index['productDefinitionTemplate']])==val)[0]
-            idxs = np.concatenate((idx_type,idx_val))
-            idx_count = 2
-        return [i[0] for i in collections.Counter(idxs).most_common() if i[1] == idx_count]
-
-
     def close(self):
         """
         Close the file handle
@@ -441,15 +418,17 @@ class open():
 
         **`leadTime : int`** specifying ending lead time (in units of hours) of a GRIB2 Message.
 
-        **`level : str`** string of value and units of the level of interest. For pressure level, use either:
-        `mb`, `pa`, or `hpa`.  For sigma levels, use `sig` or `sigma`.  For geometric height level, use `m` or `meter`
-        with optional `above ground` or `agl` [DEFAULT] or `below ground" or `bgl`.
+        **`level : str`** wgrib2-formatted layer/level string.
+
+        **`percentile : int`** specify the percentile value.
 
         **`refDate : int`** specifying the reference date in `YYYYMMDDHH[MMSS]` format.
 
         **`shortName : str`** the GRIB2 `shortName`.  This is the abbreviation name found in the NCEP GRIB2 tables.
+
+        **`threshold : str`** wgrib2-formatted probability threshold string.
         """
-        kwargs_allowed = ['duration','leadTime','level','refDate','shortName']
+        kwargs_allowed = ['duration','leadTime','level','percentile','refDate','shortName','threshold']
         idxs = {}
         for k,v in kwargs.items():
             if k not in kwargs_allowed: continue
@@ -458,11 +437,18 @@ class open():
             elif k == 'leadTime':
                 idxs[k] = np.where(np.asarray([i if i is not None else None for i in self._index['leadTime']])==v)[0]
             elif k == 'level':
-                idxs[k] = self._find_level(v)
+                idxs[k] = np.where(np.array(self._index['levelString'])==v)[0]
+            elif k == 'percentile':
+                tmp1 = np.where(np.asarray(self._index["productDefinitionTemplateNumber"])==6)[0]
+                tmp2 = np.where(np.asarray(self._index["productDefinitionTemplateNumber"])==10)[0]
+                idxs[k] = [i for i in np.concatenate((tmp1,tmp2)) if self._index["productDefinitionTemplate"][i][15]==v]
+                del tmp1,tmp2
             elif k == 'refDate':
                 idxs[k] = np.where(np.asarray(self._index['refDate'])==v)[0]
             elif k == 'shortName':
                 idxs[k] = np.where(np.array(self._index['shortName'])==v)[0]
+            elif k == 'threshold':
+                idxs[k] = np.where(np.array(self._index['probString'])==v)[0]
         idxsarr = np.concatenate(tuple(idxs.values()))
         nidxs = len(idxs.keys())
         if nidxs == 1:
@@ -480,25 +466,23 @@ class open():
 
         **`msg`**: instance of `Grib2Message`.
         """
-        self._filehandle.write(msg._msg)
-        self.size = os.path.getsize(self.name)
-        self.messages += 1
-        self.current_message += 1
+        if isinstance(msg,Grib2Message):
+            self._filehandle.write(msg._msg)
+            self.size = os.path.getsize(self.name)
+            self.messages += 1
+            self.current_message += 1
+        else:
+            raise TypeError("msg must be a Grib2Message object.")
+  
 
 class Grib2Message:
     def __init__(self, msg=None, source=None, num=-1, decode=True, discipline=None, idsect=None):
         """
-        Class Constructor
-
-        Usage
-        -----
-
-        The instantiation of this class can handle a GRIB2 message from an existing file or the
-        creation of new GRIB2 message.
-
-        To create a new GRIB2 message, provide the appropriate values to the arguments
-        `discipline` and `idsect`.  When these 2 arguments are not `None`, then a new GRIB2 message is
-        created. NOTE: All other keyword arguments are ignored when a new message is created.
+        Class Constructor. Instantiation of this class can handle a GRIB2 message from an existing 
+        file or the creation of new GRIB2 message.  To create a new GRIB2 message, provide the 
+        appropriate values to the arguments `discipline` and `idsect`.  When these 2 arguments 
+        are not `None`, then a new GRIB2 message is created. NOTE: All other keyword arguments 
+        are ignored when a new message is created.
 
         ...
 
@@ -519,19 +503,22 @@ class Grib2Message:
         **`discipline`**: integer GRIB2 Discipline [GRIB2 Table 0.0](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table0-0.shtml)
 
         **`idsect`**: Sequence containing GRIB1 Identification Section values (Section 1).
-        - idsect[0] = Id of orginating centre - [ON388 - Table 0](https://www.nco.ncep.noaa.gov/pmb/docs/on388/table0.html)
-        - idsect[1] = Id of orginating sub-centre - [ON388 - Table C](https://www.nco.ncep.noaa.gov/pmb/docs/on388/tablec.html)
-        - idsect[2] = GRIB Master Tables Version Number - [Code Table 1.0](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table1-0.shtml)
-        - idsect[3] = GRIB Local Tables Version Number - [Code Table 1.1](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table1-1.shtml)
-        - idsect[4] = Significance of Reference Time - [Code Table 1.2](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table1-2.shtml)
-        - idsect[5] = Reference Time - Year (4 digits)
-        - idsect[6] = Reference Time - Month
-        - idsect[7] = Reference Time - Day
-        - idsect[8] = Reference Time - Hour
-        - idsect[9] = Reference Time - Minute
-        - idsect[10] = Reference Time - Second
-        - idsect[11] = Production status of data - [Code Table 1.3](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table1-3.shtml)
-        - idsect[12]= Type of processed data - [Code Table 1.4](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table1-4.shtml)
+
+        | Index | Description |
+        | :---: | :---        |
+        | idsect[0] | Id of orginating centre - [ON388 - Table 0](https://www.nco.ncep.noaa.gov/pmb/docs/on388/table0.html)|
+        | idsect[1] | Id of orginating sub-centre - [ON388 - Table C](https://www.nco.ncep.noaa.gov/pmb/docs/on388/tablec.html)|
+        | idsect[2] | GRIB Master Tables Version Number - [Code Table 1.0](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table1-0.shtml)|
+        | idsect[3] | GRIB Local Tables Version Number - [Code Table 1.1](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table1-1.shtml)|
+        | idsect[4] | Significance of Reference Time - [Code Table 1.2](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table1-2.shtml)|
+        | idsect[5] | Reference Time - Year (4 digits)|
+        | idsect[6] | Reference Time - Month|
+        | idsect[7] | Reference Time - Day|
+        | idsect[8] | Reference Time - Hour|
+        | idsect[9] | Reference Time - Minute|
+        | idsect[10] | Reference Time - Second|
+        | idsect[11] | Production status of data - [Code Table 1.3](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table1-3.shtml)|
+        | idsect[12] | Type of processed data - [Code Table 1.4](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table1-4.shtml)|
         """
         self._source = source
         self._msgnum = num
@@ -543,8 +530,8 @@ class Grib2Message:
         self.isNDFD = False
         if discipline is not None and idsect is not None:
             # New message
-            self._msg,self._pos = g2clib.grib2_create(np.array([discipline,GRIB2_EDITION_NUMBER],np.int32),
-                                                      np.array(idsect,np.int32))
+            self._msg,self._pos = g2clib.grib2_create(np.array([discipline,GRIB2_EDITION_NUMBER],DEFAULT_NUMPY_INT),
+                                                      np.array(idsect,DEFAULT_NUMPY_INT))
             self._sections += [0,1]
         else:
             # Existing message
@@ -741,6 +728,8 @@ class Grib2Message:
         if self.gridDefinitionTemplateNumber.value in [0,1,203,205,32768,32769]:
             # Regular or Rotated Lat/Lon Grid
             scalefact = float(self.gridDefinitionTemplate[9])
+            if self.gridDefinitionTemplate[10] == 4294967295:
+                self.gridDefinitionTemplate[10] = -1
             divisor = float(self.gridDefinitionTemplate[10])
             if scalefact == 0: scalefact = 1.
             if divisor <= 0: divisor = 1.e6
@@ -815,6 +804,8 @@ class Grib2Message:
         elif self.gridDefinitionTemplateNumber == 40 or self.gridDefinitionTemplateNumber == 41:
             # Gaussian Grid
             scalefact = float(self.gridDefinitionTemplate[9])
+            if self.gridDefinitionTemplate[10] == 4294967295:
+                self.gridDefinitionTemplate[10] = -1
             divisor = float(self.gridDefinitionTemplate[10])
             if scalefact == 0: scalefact = 1.
             if divisor <= 0: divisor = 1.e6
@@ -919,6 +910,7 @@ class Grib2Message:
             self.unitOfSecondFixedSurface = self.typeOfSecondFixedSurface.definition[1]
             self.scaledValueOfSecondFixedSurface = self.productDefinitionTemplate[14]
             self.valueOfSecondFixedSurface = self.scaledValueOfSecondFixedSurface/(10.**self.scaleFactorOfSecondFixedSurface)
+        self.level = tables.get_wgrib2_level_string(*self.productDefinitionTemplate[9:15])
 
         # Template 4.1 -
         if self.productDefinitionTemplateNumber == 1:
@@ -935,9 +927,16 @@ class Grib2Message:
         elif self.productDefinitionTemplateNumber == 5:
             self.forecastProbabilityNumber = self.productDefinitionTemplate[15]
             self.totalNumberOfForecastProbabilities = self.productDefinitionTemplate[16]
-            self.typeOfProbability = Grib2Metadata(self.productDefinitionTemplate[16],table='4.9')
-            self.thresholdLowerLimit = self.productDefinitionTemplate[18]/(10.**self.productDefinitionTemplate[17])
-            self.thresholdUpperLimit = self.productDefinitionTemplate[20]/(10.**self.productDefinitionTemplate[19])
+            self.typeOfProbability = Grib2Metadata(self.productDefinitionTemplate[17],table='4.9')
+            self.scaleFactorOfThresholdLowerLimit = self.productDefinitionTemplate[18]
+            self.scaledValueOfThresholdLowerLimit = self.productDefinitionTemplate[19]
+            self.scaleFactorOfThresholdUpperLimit = self.productDefinitionTemplate[20]
+            self.scaledValueOfThresholdUpperLimit = self.productDefinitionTemplate[21]
+            self.thresholdLowerLimit = 0.0 if self.productDefinitionTemplate[19] == 255 else \
+                                       self.productDefinitionTemplate[19]/(10.**self.productDefinitionTemplate[18])
+            self.thresholdUpperLimit = 0.0 if self.productDefinitionTemplate[21] == 255 else \
+                                       self.productDefinitionTemplate[21]/(10.**self.productDefinitionTemplate[20])
+            self.threshold = utils.get_wgrib2_prob_string(*self.productDefinitionTemplate[17:22])
 
         # Template 4.6 -
         elif self.productDefinitionTemplateNumber == 6:
@@ -965,10 +964,15 @@ class Grib2Message:
             self.forecastProbabilityNumber = self.productDefinitionTemplate[15]
             self.totalNumberOfForecastProbabilities = self.productDefinitionTemplate[16]
             self.typeOfProbability = Grib2Metadata(self.productDefinitionTemplate[17],table='4.9')
+            self.scaleFactorOfThresholdLowerLimit = self.productDefinitionTemplate[18]
+            self.scaledValueOfThresholdLowerLimit = self.productDefinitionTemplate[19]
+            self.scaleFactorOfThresholdUpperLimit = self.productDefinitionTemplate[20]
+            self.scaledValueOfThresholdUpperLimit = self.productDefinitionTemplate[21]
             self.thresholdLowerLimit = 0.0 if self.productDefinitionTemplate[19] == 255 else \
                                        self.productDefinitionTemplate[19]/(10.**self.productDefinitionTemplate[18])
             self.thresholdUpperLimit = 0.0 if self.productDefinitionTemplate[21] == 255 else \
                                        self.productDefinitionTemplate[21]/(10.**self.productDefinitionTemplate[20])
+            self.threshold = utils.get_wgrib2_prob_string(*self.productDefinitionTemplate[17:22])
             self.yearOfEndOfTimePeriod = self.productDefinitionTemplate[22]
             self.monthOfEndOfTimePeriod = self.productDefinitionTemplate[23]
             self.dayOfEndOfTimePeriod = self.productDefinitionTemplate[24]
@@ -1041,6 +1045,12 @@ class Grib2Message:
             self.unitOfTimeRangeOfSuccessiveFields = Grib2Metadata(self.productDefinitionTemplate[29],table='4.4')
             self.timeIncrementOfSuccessiveFields = self.productDefinitionTemplate[30]
 
+        # Template 4.15 -
+        elif self.productDefinitionTemplateNumber == 15:
+            self.statisticalProcess = Grib2Metadata(self.productDefinitionTemplate[15],table='4.10')
+            self.typeOfSpatialProcessing = Grib2Metadata(self.productDefinitionTemplate[16],table='4.15')
+            self.numberOfDataPointsForSpatialProcessing = self.productDefinitionTemplate[17]
+
         else:
             if self.productDefinitionTemplateNumber != 0:
                 errmsg = 'Unsupported Product Definition Template Number - 4.%i' % self.productDefinitionTemplateNumber.value
@@ -1150,11 +1160,10 @@ class Grib2Message:
         **`masked_array`**: If `True` [DEFAULT], return masked array if there is bitmap for missing 
         or masked data.
 
-        **`expand`**: If `True` [DEFAULT], ECMWF 'reduced' gaussian grids are expanded to regular 
-        gaussian grids.
+        **`expand`**: If `True` [DEFAULT], Reduced Gaussian grids are expanded to regular Gaussian grids.
 
         **`order`**: If 0 [DEFAULT], nearest neighbor interpolation is used if grid has missing 
-        or bitmapped values. If 1, linear interpolation is used for expanding reduced gaussian grids.
+        or bitmapped values. If 1, linear interpolation is used for expanding reduced Gaussian grids.
 
         **`map_keys`**: If `True`, data values will be mapped to the string-based keys that are stored
         in the Local Use Section (section 2) of the GRIB2 Message or in a code table as specified in the
@@ -1181,9 +1190,9 @@ class Grib2Message:
             else:
                 order = 1
         drtnum = self.dataRepresentationTemplateNumber.value
-        drtmpl = np.asarray(self.dataRepresentationTemplate,dtype=np.int32)
+        drtmpl = np.asarray(self.dataRepresentationTemplate,dtype=DEFAULT_NUMPY_INT)
         gdtnum = self.gridDefinitionTemplateNumber.value
-        gdtmpl = np.asarray(self.gridDefinitionTemplate,dtype=np.int32)
+        gdtmpl = np.asarray(self.gridDefinitionTemplate,dtype=DEFAULT_NUMPY_INT)
         ndpts = self.numberOfDataPoints
         gds = self.gridDefinitionSection
         ngrdpts = gds[1]
@@ -1210,19 +1219,19 @@ class Grib2Message:
             else:
                 fld = np.reshape(fld,(self.ny,self.nx))
         else:
-            if gds[2] and gdtnum == 40: # ECMWF 'reduced' global gaussian grid.
+            if gds[2] and gdtnum == 40: # Reduced global Gaussian grid.
                 if expand:
-                    from redtoreg import _redtoreg
+                    from . import redtoreg
                     self.nx = 2*self.ny
                     lonsperlat = self.defList
                     if ma.isMA(fld):
                         fld = ma.filled(fld)
-                        fld = _redtoreg(self.nx,lonsperlat.astype(np.long),\
-                                fld.astype(np.double),fill_value)
+                        fld = redtoreg._redtoreg(self.nx,lonsperlat.astype(np.long),
+                                                 fld.astype(np.double),fill_value)
                         fld = ma.masked_values(fld,fill_value)
                     else:
-                        fld = _redtoreg(self.nx,lonsperlat.astype(np.long),\
-                                fld.astype(np.double),fill_value)
+                        fld = redtoreg._redtoreg(self.nx,lonsperlat.astype(np.long),
+                                                 fld.astype(np.double),fill_value)
         # Check scan modes for rect grids.
         if self.nx is not None and self.ny is not None:
             if self.scanModeFlags[3]:
@@ -1239,7 +1248,8 @@ class Grib2Message:
                self.identificationSection[1] == 14 and \
                self.shortName == 'PWTHER':
                 # MDL Predominant Weather Grid
-                keys = utils.decode_mdl_wx_strings(self._lus)
+                #keys = utils.decode_mdl_wx_strings(self._lus)
+                keys = utils.decode_ndfd_wx_strings(self._lus)
                 for n,k in enumerate(keys):
                     fld = np.where(fld==str(n+1),k,fld)
             elif self.identificationSection[0] == 8 and \
@@ -1265,7 +1275,7 @@ class Grib2Message:
     def grid(self):
         """
         Return lats,lons (in degrees) of grid. Currently can handle reg. lat/lon, 
-        global gaussian, mercator, stereographic, lambert conformal, albers equal-area, 
+        global Gaussian, mercator, stereographic, lambert conformal, albers equal-area, 
         space-view and azimuthal equidistant grids.
 
         Returns
@@ -1297,23 +1307,20 @@ class Grib2Message:
             #    lats = lats[::-1]
             self.projparams['proj'] = 'cyl'
             lons,lats = np.meshgrid(lons,lats) # make 2-d arrays.
-        elif gdtnum == 40: # gaussian grid (only works for global!)
-            try:
-                from pygrib import gaulats
-            except:
-                raise ImportError("pygrib required to compute Gaussian latitude")
+        elif gdtnum == 40: # Gaussian grid (only works for global!)
+            from utils.gauss_grids import gaussian_latitudes
             lon1, lat1 = self.longitudeFirstGridpoint, self.latitudeFirstGridpoint
             lon2, lat2 = self.longitudeLastGridpoint, self.latitudeLastGridpoint
             nlats = self.ny
-            if not reggrid: # ECMWF 'reduced' gaussian grid.
+            if not reggrid: # Reduced Gaussian grid.
                 nlons = 2*nlats
                 dlon = 360./nlons
             else:
                 nlons = self.nx
                 dlon = self.gridlengthXDirection
             lons = np.arange(lon1,lon2+dlon,dlon)
-            # Compute gaussian lats (north to south)
-            lats = gaulats(nlats)
+            # Compute Gaussian lats (north to south)
+            lats = gaussian_latitudes(nlats)
             if lat1 < lat2:  # reverse them if necessary
                 lats = lats[::-1]
             # flip if scan mode says to.
@@ -1419,11 +1426,14 @@ class Grib2Message:
         ----------
 
         **`gdsinfo`**: Sequence containing information needed for the grid definition section.
-        - gdsinfo[0] = Source of grid definition - [Code Table 3.0](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table3-0.shtml)
-        - gdsinfo[1] = Number of data points
-        - gdsinfo[2] = Number of octets for optional list of numbers defining number of points
-        - gdsinfo[3] = Interpetation of list of numbers defining number of points - [Code Table 3.11](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table3-11.shtml)
-        - gdsinfo[4] = Grid Definition Template Number - [Code Table 3.1](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table3-1.shtml)
+
+        | Index | Description |
+        | :---: | :---        |
+        | gdsinfo[0] | Source of grid definition - [Code Table 3.0](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table3-0.shtml)|
+        | gdsinfo[1] | Number of data points|
+        | gdsinfo[2] | Number of octets for optional list of numbers defining number of points|
+        | gdsinfo[3] | Interpetation of list of numbers defining number of points - [Code Table 3.11](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table3-11.shtml)|
+        | gdsinfo[4] | Grid Definition Template Number - [Code Table 3.1](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table3-1.shtml)|
 
         **`gdtmpl`**: Sequence of values for the specified Grid Definition Template. Each 
         element of this integer array contains an entry (in the order specified) of Grid
@@ -1435,7 +1445,7 @@ class Grib2Message:
         if 3 in self._sections:
             raise ValueError('GRIB2 Message already contains Grid Definition Section.')
         if deflist is not None:
-            _deflist = np.array(deflist,dtype=np.int32)
+            _deflist = np.array(deflist,dtype=DEFAULT_NUMPY_INT)
         else:
             _deflist = None
         gdtnum = gdsinfo[4]
@@ -1460,13 +1470,13 @@ class Grib2Message:
         elif gdtnum in [1000,1100]:
             self.scanModeFlags = utils.int2bin(gdtmpl[12],output=list)[0:4]
         self._msg,self._pos = g2clib.grib2_addgrid(self._msg,
-                                                   np.array(gdsinfo,dtype=np.int32),
-                                                   np.array(gdtmpl,dtype=np.int32),
+                                                   np.array(gdsinfo,dtype=DEFAULT_NUMPY_INT),
+                                                   np.array(gdtmpl,dtype=DEFAULT_NUMPY_INT),
                                                    _deflist)
         self._sections.append(3)
 
 
-    def addfield(self, pdtnum, pdtmpl, drtnum, drtmpl, field, coordlist=None):
+    def addfield(self, field, pdtnum, pdtmpl, coordlist=None, packing="complex-spdiff", **packing_opts): 
         """
         Add a Product Definition, Data Representation, Bitmap, and Data Sections 
         to `Grib2Message` instance (i.e. Sections 4-7).  Must be called after the grid 
@@ -1475,37 +1485,53 @@ class Grib2Message:
         Parameters
         ----------
 
+        **`field`**: Numpy array of data values to pack.  If field is a masked array, then 
+        a bitmap is created from the mask.
+
         **`pdtnum`**: integer Product Definition Template Number - [Code Table 4.0](http://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table4-0.shtml)
 
         **`pdtmpl`**: Sequence with the data values for the specified Product Definition 
         Template (N=pdtnum).  Each element of this integer array contains an entry (in 
         the order specified) of Product Definition Template 4.N.
 
-        **`drtnum`**: integer Data Representation Template Number - [Code Table 5.0](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_table5-0.shtml)
-
-        **`drtmpl`**: Sequence with the data values for the specified Data Representation
-        Template (N=drtnum).  Each element of this integer array contains an entry (in 
-        the order specified) of Data Representation Template 5.N.  Note that some values 
-        in this template (eg. reference values, number of bits, etc...) may be changed by the
-        data packing algorithms.  Use this to specify scaling factors and order of spatial 
-        differencing, if desired.
-
-        **`field`**: Numpy array of data points to pack.  If field is a masked array, then 
-        a bitmap is created from the mask.
-
         **`coordlist`**: Sequence containing floating point values intended to document the 
         vertical discretization with model data on hybrid coordinate vertical levels. Default is `None`.
+
+        **`packing`**: String to specify the type of packing. Valid options are the following:
+
+        | Packing Scheme | Description |
+        | :---:          | :---:       |
+        | 'simple'         | [Simple packing](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_temp5-0.shtml) | 
+        | 'complex'        | [Complex packing](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_temp5-2.shtml) | 
+        | 'complex-spdiff' | [Complex packing with Spatial Differencing](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_temp5-3.shtml) | 
+        | 'jpeg'           | [JPEG compression](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_temp5-40.shtml) | 
+        | 'png'            | [PNG compression](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_temp5-41.shtml) | 
+        | 'spectral-simple'| [Spectral Data - Simple packing](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_temp5-50.shtml) | 
+        | 'spectral-complex'| [Spectral Data - Complex packing](https://www.nco.ncep.noaa.gov/pmb/docs/grib2/grib2_doc/grib2_temp5-51.shtml) | 
+
+        **`**packing_opts`**: Packing keyword arguments. The keywords are the same as Grib2Message attribute names for
+        the Data Representation Template (Section 5) metadata. Valid keywords per packing scheme are the following:
+
+        | Packing Scheme | Keyword Arguments |
+        | :---:          | :---:                      |
+        | 'simple'     | `binScaleFactor`, `decScaleFactor` |
+        | 'complex'     | `binScaleFactor`, `decScaleFactor`, `priMissingValue`, [`secMissingValue`] |
+        | 'complex-spdiff'     | `binScaleFactor`, `decScaleFactor`, `spatialDifferenceOrder`, `priMissingValue`, [`secMissingValue`] |
+        | 'jpeg'     | `binScaleFactor`, `decScaleFactor` |
+        | 'png'     | `binScaleFactor`, `decScaleFactor` |
+        | 'spectral-simple'     | `binScaleFactor`, `decScaleFactor` |
+        | 'spectral-complex'     | `binScaleFactor`, `decScaleFactor` |
         """
-        if not hasattr(self,'scanModeFlags'):
+        if self._sections[-1] != 3:
             raise ValueError('addgrid() must be called before addfield()')
         if self.scanModeFlags is not None:
             if self.scanModeFlags[3]:
                 fieldsave = field.astype('f') # Casting makes a copy
                 field[1::2,:] = fieldsave[1::2,::-1]
         fld = field.astype('f')
-        if ma.isMA(field):
-            bmap = 1-np.ravel(field.mask.astype('i'))
+        if ma.isMA(field) and ma.count_masked(field) > 0:
             bitmapflag = 0
+            bmap = 1-np.ravel(field.mask.astype(DEFAULT_NUMPY_INT))
         else:
             bitmapflag = 255
             bmap = None
@@ -1513,14 +1539,57 @@ class Grib2Message:
             crdlist = np.array(coordlist,'f')
         else:
             crdlist = None
-        _pdtnum = pdtnum.value if isinstance(pdtnum,Grib2Metadata) else pdtnum
-        _drtnum = drtnum.value if isinstance(drtnum,Grib2Metadata) else drtnum
+
+        # Set data representation template number and template values
+        drtnum = -1
+        drtmpl = np.zeros((DEFAULT_DRT_LEN),dtype=DEFAULT_NUMPY_INT)
+        if packing == "simple":
+            drtnum = 0
+            drtmpl[1] = packing_opts["binScaleFactor"]
+            drtmpl[2] = packing_opts["decScaleFactor"]
+        elif packing == "complex" or packing == "complex-spdiff":
+            if packing == "complex":
+                drtnum = 2
+            if packing == "complex-spdiff":
+                drtnum = 3
+                drtmpl[16] = packing_opts['spatialDifferenceOrder']
+            drtmpl[1] = packing_opts["binScaleFactor"]
+            drtmpl[2] = packing_opts["decScaleFactor"]
+            if set(("priMissingValue","secMissingValue")).issubset(packing_opts):
+                drtmpl[6] = 2
+                drtmpl[7] = utils.putieeeint(packing_opts["priMissingValue"])
+                drtmpl[8] = utils.putieeeint(packing_opts["secMissingValue"])
+            else:
+                if "priMissingValue" in packing_opts.keys():
+                    drtmpl[6] = 1
+                    drtmpl[7] = utils.putieeeint(packing_opts["priMissingValue"])
+                else:
+                    drtmpl[6] = 0
+        elif packing == "jpeg":
+            drtnum = 40
+            drtmpl[1] = packing_opts["binScaleFactor"]
+            drtmpl[2] = packing_opts["decScaleFactor"]
+        elif packing == "png":
+            drtnum = 41
+            drtmpl[1] = packing_opts["binScaleFactor"]
+            drtmpl[2] = packing_opts["decScaleFactor"]
+        elif packing == "spectral-simple":
+            drtnum = 50
+            drtmpl[1] = packing_opts["binScaleFactor"]
+            drtmpl[2] = packing_opts["decScaleFactor"]
+        elif packing == "spectral-complex":
+            drtnum = 51
+            drtmpl[1] = packing_opts["binScaleFactor"]
+            drtmpl[2] = packing_opts["decScaleFactor"]
+
+        pdtnum = pdtnum.value if isinstance(pdtnum,Grib2Metadata) else pdtnum
+
         self._msg,self._pos = g2clib.grib2_addfield(self._msg,
-                                                    _pdtnum,
-                                                    np.array(pdtmpl,dtype=np.int32),
+                                                    pdtnum,
+                                                    np.array(pdtmpl,dtype=DEFAULT_NUMPY_INT),
                                                     crdlist,
-                                                    _drtnum,
-                                                    np.array(drtmpl,dtype=np.int32),
+                                                    drtnum,
+                                                    drtmpl,
                                                     np.ravel(fld),
                                                     bitmapflag,
                                                     bmap)
