@@ -3,8 +3,8 @@ table_4_2_0_0  = {
 '1':['Virtual Temperature','K','VTMP'],
 '2':['Potential Temperature','K','POT'],
 '3':['Pseudo-Adiabatic Potential Temperature (or Equivalent Potential Temperature)','K','EPOT'],
-'4':['Maximum Temperature*','K','TMAX'],
-'5':['Minimum Temperature*','K','TMIN'],
+'4':['Maximum Temperature','K','TMAX'],
+'5':['Minimum Temperature','K','TMIN'],
 '6':['Dew Point Temperature','K','DPT'],
 '7':['Dew Point Depression (or Deficit)','K','DEPR'],
 '8':['Lapse Rate','K m-1','LAPR'],
@@ -13,14 +13,14 @@ table_4_2_0_0  = {
 '11':['Sensible Heat Net Flux','W m-2','SHTFL'],
 '12':['Heat Index','K','HEATX'],
 '13':['Wind Chill Factor','K','WCF'],
-'14':['Minimum Dew Point Depression*','K','MINDPD'],
+'14':['Minimum Dew Point Depression','K','MINDPD'],
 '15':['Virtual Potential Temperature','K','VPTMP'],
 '16':['Snow Phase Change Heat Flux','W m-2','SNOHF'],
 '17':['Skin Temperature','K','SKINT'],
 '18':['Snow Temperature (top of snow)','K','SNOT'],
 '19':['Turbulent Transfer Coefficient for Heat','Numeric','TTCHT'],
 '20':['Turbulent Diffusion Coefficient for Heat','m2s-1','TDCHT'],
-'21':['Apparent Temperature **','K','APTMP'],
+'21':['Apparent Temperature','K','APTMP'],
 '22':['Temperature Tendency due to Short-Wave Radiation','K s-1','TTSWR'],
 '23':['Temperature Tendency due to Long-Wave Radiation','K s-1','TTLWR'],
 '24':['Temperature Tendency due to Short-Wave Radiation,  Clear Sky','K s-1','TTSWRCS'],
@@ -29,7 +29,10 @@ table_4_2_0_0  = {
 '27':['Wet Bulb Temperature','K','WETBT'],
 '28':['Unbalanced Component of Temperature','K','UCTMP'],
 '29':['Temperature Advection','K s-1','TMPADV'],
-'30-191':['Reserved','unknown','unknown'],
+'30':['Latent Heat Net Flux Due to Evaporation','W m-2','LHFLXE'],
+'31':['Latent Heat Net Flux Due to Sublimation','W m-2','LHFLXS'],
+'32':['Wet-Bulb Potential Temperature','K','WETBPT'],
+'33-191':['Reserved','unknown','unknown'],
 '192-254':['Reserved for Local Use','unknown','unknown'],
 '192':['Snow Phase Change Heat Flux','W m-2','SNOHF'],
 '193':['Temperature Tendency by All Radiation','K s-1','TTRAD'],
@@ -56,15 +59,15 @@ table_4_2_0_1  = {
 '4':['Vapour Pressure','Pa','VAPP'],
 '5':['Saturation Deficit','Pa','SATD'],
 '6':['Evaporation','kg m-2','EVP'],
-'7':['Precipitation Rate *','kg m-2 s-1','PRATE'],
-'8':['Total Precipitation ***','kg m-2','APCP'],
-'9':['Large-Scale Precipitation (non-convective) ***','kg m-2','NCPCP'],
-'10':['Convective Precipitation ***','kg m-2','ACPCP'],
+'7':['Precipitation Rate','kg m-2 s-1','PRATE'],
+'8':['Total Precipitation','kg m-2','APCP'],
+'9':['Large-Scale Precipitation (non-convective)','kg m-2','NCPCP'],
+'10':['Convective Precipitation','kg m-2','ACPCP'],
 '11':['Snow Depth','m','SNOD'],
-'12':['Snowfall Rate Water Equivalent *','kg m-2 s-1','SRWEQ'],
-'13':['Water Equivalent of Accumulated Snow Depth ***','kg m-2','WEASD'],
-'14':['Convective Snow ***','kg m-2','SNOC'],
-'15':['Large-Scale Snow ***','kg m-2','SNOL'],
+'12':['Snowfall Rate Water Equivalent','kg m-2 s-1','SRWEQ'],
+'13':['Water Equivalent of Accumulated Snow Depth','kg m-2','WEASD'],
+'14':['Convective Snow','kg m-2','SNOC'],
+'15':['Large-Scale Snow','kg m-2','SNOL'],
 '16':['Snow Melt','kg m-2','SNOM'],
 '17':['Snow Age','day','SNOAG'],
 '18':['Absolute Humidity','kg m-3','ABSH'],
@@ -76,9 +79,9 @@ table_4_2_0_1  = {
 '24':['Rain Mixing Ratio','kg kg-1','RWMR'],
 '25':['Snow Mixing Ratio','kg kg-1','SNMR'],
 '26':['Horizontal Moisture Convergence','kg kg-1 s-1','MCONV'],
-'27':['Maximum Relative Humidity *','%','MAXRH'],
-'28':['Maximum Absolute Humidity *','kg m-3','MAXAH'],
-'29':['Total Snowfall ***','m','ASNOW'],
+'27':['Maximum Relative Humidity','%','MAXRH'],
+'28':['Maximum Absolute Humidity','kg m-3','MAXAH'],
+'29':['Total Snowfall','m','ASNOW'],
 '30':['Precipitable Water Category','See Table 4.202','PWCAT'],
 '31':['Hail','m','HAIL'],
 '32':['Graupel','kg kg-1','GRLE'],
@@ -96,13 +99,13 @@ table_4_2_0_1  = {
 '44':['Rime Factor','Numeric','RIME'],
 '45':['Total Column Integrated Rain','kg m-2','TCOLR'],
 '46':['Total Column Integrated Snow','kg m-2','TCOLS'],
-'47':['Large Scale Water Precipitation (Non-Convective) ***','kg m-2','LSWP'],
-'48':['Convective Water Precipitation ***','kg m-2','CWP'],
-'49':['Total Water Precipitation ***','kg m-2','TWATP'],
-'50':['Total Snow Precipitation ***','kg m-2','TSNOWP'],
+'47':['Large Scale Water Precipitation (Non-Convective)','kg m-2','LSWP'],
+'48':['Convective Water Precipitation','kg m-2','CWP'],
+'49':['Total Water Precipitation','kg m-2','TWATP'],
+'50':['Total Snow Precipitation','kg m-2','TSNOWP'],
 '51':['Total Column Water  (Vertically integrated total water (vapour+cloud water/ice)','kg m-2','TCWAT'],
-'52':['Total Precipitation Rate **','kg m-2 s-1','TPRATE'],
-'53':['Total Snowfall Rate Water Equivalent **','kg m-2 s-1','TSRWE'],
+'52':['Total Precipitation Rate','kg m-2 s-1','TPRATE'],
+'53':['Total Snowfall Rate Water Equivalent','kg m-2 s-1','TSRWE'],
 '54':['Large Scale Precipitation Rate','kg m-2 s-1','LSPRATE'],
 '55':['Convective Snowfall Rate Water Equivalent','kg m-2 s-1','CSRWE'],
 '56':['Large Scale Snowfall Rate Water Equivalent','kg m-2 s-1','LSSRWE'],
@@ -171,7 +174,35 @@ table_4_2_0_1  = {
 '119':['Unbalanced Component of Specific Cloud Liquid Water content','kg kg-1','UCSCLW'],
 '120':['Unbalanced Component of Specific Cloud Ice Water content','kg kg-1','UCSCIW'],
 '121':['Fraction of Snow Cover','Proportion','FSNOWC'],
-'122-128':['Reserved','unknown','unknown'],
+'122':['Precipitation intensity index','See Table 4.247','PIIDX'],
+'123':['Domiunknownt precipitation type','See Table 4.201','DPTYPE'],
+'124':['Presence of showers','See Table 4.222','PSHOW'],
+'125':['Presence of blowing snow','See Table 4.222','PBSNOW'],
+'126':['Presence of blizzard','See Table 4.222','PBLIZZ'],
+'127':['Ice pellets (non-water equivalent) precipitation rate','m s-1','ICEP'],
+'128':['Total solid precipitation rate','kg m-2 s-1','TSPRATE'],
+'129':['Effective radius of cloud water','m','ERADCW'],
+'130':['Effective radius of rain','m','ERADRAIN'],
+'131':['Effective radius of cloud ice','m','ERADCICE'],
+'132':['Effective radius of snow','m','ERADSNOW'],
+'133':['Effective radius of graupel','m','ERADGRL'],
+'134':['Effective radius of hail','m','ERADHAIL'],
+'135':['Effective radius of subgrid liquid clouds','m','ERADSLC'],
+'136':['Effective radius of subgrid ice clouds','m','ERADSIC'],
+'137':['Effective aspect ratio of rain','unknown','ERATIOR'],
+'138':['Effective aspect ratio of cloud ice','unknown','ERATIOCI'],
+'139':['Effective aspect ratio of snow','unknown','ERATIOS'],
+'140':['Effective aspect ratio of graupel','unknown','ERATIOG'],
+'141':['Effective aspect ratio of hail','unknown','ERATIOH'],
+'142':['Effective ratio of subgrid ice clouds','unknown','ERATOSIC'],
+'143':['Potential evaporation rate','kg m-2 s-1','PERATE'],
+'144':['Specific rain water content (convective)','kg kg-1','SRWATERC'],
+'145':['Specific snow water content (convective)','kg kg-1','SSNOWWC'],
+'146':['Cloud ice precipitation rate','kg m-2 s-1','CICEPR'],
+'147':['Character of precipitation','See Table 4.249','PERATE'],
+'148':['Snow evaporation rate','kg m-2 s-1','SNOWERAT'],
+'149':['Cloud water mixing ratio','kg kg-1','CWATERMR'],
+'150-128':['Reserved','unknown','unknown'],
 '129':['Effective Radius of Cloud Water','m','EFRCWAT'],
 '130':['Effective Radius of Rain','m','EFRRAIN'],
 '131':['Effective Radius of Cloud Ice','m','EFRCICE'],
@@ -260,7 +291,7 @@ table_4_2_0_2  = {
 '18':['Momentum Flux, V-Component','N m-2','VFLX'],
 '19':['Wind Mixing Energy','J','WMIXE'],
 '20':['Boundary Layer Dissipation','W m-2','BLYDP'],
-'21':['Maximum Wind Speed *','m s-1','MAXGUST'],
+'21':['Maximum Wind Speed','m s-1','MAXGUST'],
 '22':['Wind Speed (Gust)','m s-1','GUST'],
 '23':['U-Component of Wind (Gust)','m s-1','UGUST'],
 '24':['V-Component of Wind (Gust)','m s-1','VGUST'],
@@ -273,11 +304,11 @@ table_4_2_0_2  = {
 '31':['Turbulent Diffusion Coefficient for Momentum','m2 s-1','TDCMOM'],
 '32':['Eta Coordinate Vertical Velocity','s-1','ETACVV'],
 '33':['Wind Fetch','m','WINDF'],
-'34':['Normal Wind Component **','m s-1','NWIND'],
-'35':['Tangential Wind Component **','m s-1','TWIND'],
-'36':['Amplitude Function for Rossby Wave Envelope for Meridional Wind ***','m s-1','AFRWE'],
-'37':['Northward Turbulent Surface Stress ****','N m-2 s','NTSS'],
-'38':['Eastward Turbulent Surface Stress ****','N m-2 s','ETSS'],
+'34':['Normal Wind Component','m s-1','NWIND'],
+'35':['Tangential Wind Component','m s-1','TWIND'],
+'36':['Amplitude Function for Rossby Wave Envelope for Meridional Wind','m s-1','AFRWE'],
+'37':['Northward Turbulent Surface Stress','N m-2 s','NTSS'],
+'38':['Eastward Turbulent Surface Stress','N m-2 s','ETSS'],
 '39':['Eastward Wind Tendency Due to Parameterizations','m s-2','EWTPARM'],
 '40':['Northward Wind Tendency Due to Parameterizations','m s-2','NWTPARM'],
 '41':['U-Component of Geostrophic Wind','m s-1','UGWIND'],
@@ -286,7 +317,9 @@ table_4_2_0_2  = {
 '44':['Geostrophic Wind Speed','m s-1','GEOWS'],
 '45':['Unbalanced Component of Divergence','s-1','UNDIV'],
 '46':['Vorticity Advection','s-2','VORTADV'],
-'47-191':['Reserved','unknown','unknown'],
+'47':['Surface roughness for heat,(see Note 5)','m','SFRHEAT'],
+'48':['Surface roughness for moisture,(see Note 6)','m','SFRMOIST'],
+'49-191':['Reserved','unknown','unknown'],
 '192-254':['Reserved for Local Use','unknown','unknown'],
 '192':['Vertical Speed Shear','s-1','VWSH'],
 '193':['Horizontal Momentum Flux','N m-2','MFLX'],
@@ -369,7 +402,10 @@ table_4_2_0_3  = {
 '29':['Updraught Detrainment Rate','kg m-3 s-1','UDRATE'],
 '30':['Downdraught Detrainment Rate','kg m-3 s-1','DDRATE'],
 '31':['Unbalanced Component of Logarithm of Surface Pressure','unknown','UCLSPRS'],
-'32-191':['Reserved','unknown','unknown'],
+'32':['Saturation water vapour pressure','Pa','SWATERVP'],
+'33':['Geometric altitude above mean sea level','m','GAMSL'],
+'34':['Geometric height above ground level','m','GHAGRD'],
+'35-191':['Reserved','unknown','unknown'],
 '192-254':['Reserved for Local Use','unknown','unknown'],
 '192':['MSLP (Eta model reduction)','Pa','MSLET'],
 '193':['5-Wave Geopotential Height','gpm','5WAVH'],
@@ -405,9 +441,9 @@ table_4_2_0_3  = {
 '255':['Missing','unknown','unknown'],
 }
 table_4_2_0_4  = {
-'0':['Net Short-Wave Radiation Flux (Surface)*','W m-2','NSWRS'],
-'1':['Net Short-Wave Radiation Flux (Top of Atmosphere)*','W m-2','NSWRT'],
-'2':['Short-Wave Radiation Flux*','W m-2','SWAVR'],
+'0':['Net Short-Wave Radiation Flux (Surface)','W m-2','NSWRS'],
+'1':['Net Short-Wave Radiation Flux (Top of Atmosphere)','W m-2','NSWRT'],
+'2':['Short-Wave Radiation Flux','W m-2','SWAVR'],
 '3':['Global Radiation Flux','W m-2','GRAD'],
 '4':['Brightness Temperature','K','BRTMP'],
 '5':['Radiance (with respect to wave number)','W m-1 sr-1','LWRAD'],
@@ -421,11 +457,12 @@ table_4_2_0_4  = {
 '13':['Direct Short Wave Radiation Flux','W m-2','DSWRFLX'],
 '14':['Diffuse Short Wave Radiation Flux','W m-2','DIFSWRF'],
 '15-49':['Reserved','unknown','unknown'],
-'50':['UV Index (Under Clear Sky)**','Numeric','UVIUCS'],
-'51':['UV Index**','W m-2','UVI'],
+'50':['UV Index (Under Clear Sky)','Numeric','UVIUCS'],
+'51':['UV Index','W m-2','UVI'],
 '52':['Downward Short-Wave Radiation Flux, Clear Sky','W m-2','DSWRFCS'],
 '53':['Upward Short-Wave Radiation Flux, Clear Sky','W m-2','USWRFCS'],
-'54-191':['Reserved','unknown','unknown'],
+'54':['Direct normal short-wave radiation flux,(see Note 3)','W m-2','DSWRFLX'],
+'55-191':['Reserved','unknown','unknown'],
 '192-254':['Reserved for Local Use','unknown','unknown'],
 '192':['Downward Short-Wave Radiation Flux','W m-2','DSWRF'],
 '193':['Upward Short-Wave Radiation Flux','W m-2','USWRF'],
@@ -444,9 +481,9 @@ table_4_2_0_4  = {
 '255':['Missing','unknown','unknown'],
 }
 table_4_2_0_5  = {
-'0':['Net Long-Wave Radiation Flux (Surface)*','W m-2','NLWRS'],
-'1':['Net Long-Wave Radiation Flux (Top of Atmosphere)*','W m-2','NLWRT'],
-'2':['Long-Wave Radiation Flux*','W m-2','LWAVR'],
+'0':['Net Long-Wave Radiation Flux (Surface)','W m-2','NLWRS'],
+'1':['Net Long-Wave Radiation Flux (Top of Atmosphere)','W m-2','NLWRT'],
+'2':['Long-Wave Radiation Flux','W m-2','LWAVR'],
 '3':['Downward Long-Wave Rad. Flux','W m-2','DLWRF'],
 '4':['Upward Long-Wave Rad. Flux','W m-2','ULWRF'],
 '5':['Net Long-Wave Radiation Flux','W m-2','NLWRF'],
@@ -481,13 +518,13 @@ table_4_2_0_6  = {
 '14':['Non-Convective Cloud Cover','%','CDLYR'],
 '15':['Cloud Work Function','J kg-1','CWORK'],
 '16':['Convective Cloud Efficiency','Proportion','CUEFI'],
-'17':['Total Condensate * - Parameter deprecated','kg kg-1','TCONDO'],
-'18':['Total Column-Integrated Cloud Water * - Parameter deprecated','kg m-2','TCOLWO'],
-'19':['Total Column-Integrated Cloud Ice * - Parameter deprecated','kg m-2','TCOLIO'],
-'20':['Total Column-Integrated Condensate * - Parameter deprecated','kg m-2','TCOLC'],
+'17':['Total Condensate','kg kg-1','TCONDO'],
+'18':['Total Column-Integrated Cloud Water','kg m-2','TCOLWO'],
+'19':['Total Column-Integrated Cloud Ice','kg m-2','TCOLIO'],
+'20':['Total Column-Integrated Condensate','kg m-2','TCOLC'],
 '21':['Ice fraction of total condensate','Proportion','FICE'],
 '22':['Cloud Cover','%','CDCC'],
-'23':['Cloud Ice Mixing Ratio * - Parameter deprecated','kg kg-1','CDCIMR'],
+'23':['Cloud Ice Mixing Ratio','kg kg-1','CDCIMR'],
 '24':['Sunshine','Numeric','SUNS'],
 '25':['Horizontal Extent of Cumulonimbus (CB)','%','CBHE'],
 '26':['Height of Convective Cloud Base','m','HCONCB'],
@@ -506,9 +543,9 @@ table_4_2_0_6  = {
 '39':['Mass Density of Cloud Ice','kg m-3','MASSDCI'],
 '40':['Mass Density of Convective Cloud Water Droplets','kg m-3','MDCCWD'],
 '41-46':['Reserved','unknown','unknown'],
-'47':['Volume Fraction of Cloud Water Droplets **','Numeric','VFRCWD'],
-'48':['Volume Fraction of Cloud Ice Particles **','Numeric','VFRCICE'],
-'49':['Volume Fraction of Cloud (Ice and/or Water) **','Numeric','VFRCIW'],
+'47':['Volume Fraction of Cloud Water Droplets','Numeric','VFRCWD'],
+'48':['Volume Fraction of Cloud Ice Particles','Numeric','VFRCICE'],
+'49':['Volume Fraction of Cloud (Ice and/or Water)','Numeric','VFRCIW'],
 '50-191':['Reserved','unknown','unknown'],
 '192-254':['Reserved for Local Use','unknown','unknown'],
 '192':['Non-Convective Cloud Cover','%','CDLYR'],
@@ -544,7 +581,8 @@ table_4_2_0_7  = {
 '17':['Gradient Richardson Number','Numeric','GRDRN'],
 '18':['Flux Richardson Number','Numeric','FLXRN'],
 '19':['Convective Available Potential Energy Shear','m2 s-2','CONAPES'],
-'20-191':['Reserved','unknown','unknown'],
+'20':['Thunderstorm intensity index','See Table 4.246','TIIDEX'],
+'21-191':['Reserved','unknown','unknown'],
 '192-254':['Reserved for Local Use','unknown','unknown'],
 '192':['Surface Lifted Index','K','LFT X'],
 '193':['Best (4 layer) Lifted Index','K','4LFTX'],
@@ -646,7 +684,8 @@ table_4_2_0_17  = {
 '2':['Cloud-to-Ground Lightning Flash Density','km-2 day-1','CDGDLTFD'],
 '3':['Cloud-to-Cloud Lightning Flash Density','km-2 day-1','CDCDLTFD'],
 '4':['Total Lightning Flash Density','km-2 day-1','TLGTFD'],
-'5-191':['Reserved','unknown','unknown'],
+'5':['Subgrid-scale lightning potential index','J kg-1','SLNGPIDX'],
+'6-191':['Reserved','unknown','unknown'],
 '192-254':['Reserved for Local Use','unknown','unknown'],
 '192':['Lightning','non-dim','LTNG'],
 '255':['Missing','unknown','unknown'],
@@ -714,7 +753,10 @@ table_4_2_0_19  = {
 '35':['Visibility Through Blowing Snow','m','VISBSN'],
 '36':['Presence of Snow Squalls','See Table 4.222','PSNOWS'],
 '37':['Icing Severity','See Table 4.228','ICESEV'],
-'38-191':['Reserved','unknown','unknown'],
+'38':['Sky transparency index','See Table 4.214','SKYIDX'],
+'39':['Seeing index','See Table 4.214','SEEINDEX'],
+'40':['Snow level','m','SNOWLVL'],
+'41-191':['Reserved','unknown','unknown'],
 '192-254':['Reserved for Local Use','unknown','unknown'],
 '192':['Maximum Snow Albedo','%','MXSALB'],
 '193':['Snow-Free Albedo','%','SNFALB'],
